library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ButtonFilter is
	generic(	CLK_FREQ			: integer;
				NUM_SWITCHES	: integer);
	port (	clk				: in	std_logic;
				reset				: in	std_logic;
				buttonArray		: in	std_logic_vector(NUM_SWITCHES-1 downto 0);
				pressArray		: out	std_logic_vector(NUM_SWITCHES-1 downto 0));

end entity ButtonFilter;

architecture Arch of ButtonFilter is

	constant TICK_COUNT_MAX	: integer := CLK_FREQ / 1000;
	signal tickCount			: integer range 0 to TICK_COUNT_MAX;
	signal tick					: std_logic;

begin

tick <= '1' when (tickCount=0) else '0';

Timer : process(clk)
begin
	if (clk'event and clk='1') then
		if (reset='1' or tickCount=0) then
			tickCount <= TICK_COUNT_MAX;
		else
			tickCount <= tickCount - 1;
		end if;
	end if;
end process Timer;

ButtonGenerate : for buttonNum in 0 to NUM_SWITCHES-1 generate
	signal syncReg		: std_logic_vector(3 downto 0);
	signal counter		: unsigned(3 downto 0);

	signal state		: std_logic;
	signal prevState	: std_logic;
begin

	ButtonControl : process(clk)
	begin
		if (clk'event and clk='1') then
			if (reset='1') then
				syncReg <= (others => '0');
				counter <= (others => '0');
				state <= '0';
				prevState <= '0';

			else
				-- Syncronise the button signal to the clk
				syncReg <= syncReg(2 downto 0) & not buttonArray(buttonNum);

				-- Debounce the signal
				if (syncReg(3)='0') then
					counter <= (others => '0');
					state <= '0';

				elsif (tick='1') then
					if (counter=b"1111") then
						state <= '1';
					else
						counter <= counter + 1;
					end if;
				end if;
		
				-- Check for a press event
				if (prevState='0' and state='1') then
					pressArray(buttonNum) <= '1';
				else
					pressArray(buttonNum) <= '0';
				end if;

				prevState <= state;

			end if;
		end if;
	end process ButtonControl;

end generate;

end Arch;
