------------------------------------------------------------------------------
--				Width		Write						Read
-- Reg 0		(2)		config Array Num		config Array Num		+ 0
-- Reg 1		(5)		config Cell Num		config Cell Num		+ 4
-- Reg 2		(12)		config												+ 8	(Increments config cell num)

-- Reg 3		(5 + 5)	outputSel 1 & 0									+ 12

-- Reg 4		(2)		failMode Array Num	failMode Array Num	+ 16
-- Reg 5		(5)		failMode Cell Num		failMode Cell Num		+ 20
-- Reg 6		(2)		failMode												+ 24

-- Reg 7		(3)		inputSel					inputSel					+ 28
-- Reg 8		(4)		vector					vector					+ 32
-- Reg 9		(2)		ramData					ramData					+ 36	(Increments vector)

-- Reg A		(6 + 6)								output & match			+ 40
-- Reg B		(3)		errorReset				error						+ 44
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity opb_ftblock is
generic
(
	C_BASEADDR		: std_logic_vector	:= X"00000000";
	C_HIGHADDR		: std_logic_vector	:= X"0000000F";
	C_OPB_AWIDTH	: integer				:= 32;
	C_OPB_DWIDTH	: integer				:= 32;
	C_FAMILY			: string					:= "virtex2p"
);
port
(
	ledPort			: out	std_logic_vector(2 downto 0);

	OPB_Clk			: in	std_logic;
	OPB_Rst			: in	std_logic;
	Sl_DBus			: out	std_logic_vector(0 to C_OPB_DWIDTH-1);
	Sl_errAck		: out	std_logic;
	Sl_retry			: out	std_logic;
	Sl_toutSup		: out	std_logic;
	Sl_xferAck		: out	std_logic;
	OPB_ABus			: in	std_logic_vector(0 to C_OPB_AWIDTH-1);
	OPB_BE			: in	std_logic_vector(0 to C_OPB_DWIDTH/8-1);
	OPB_DBus			: in	std_logic_vector(0 to C_OPB_DWIDTH-1);
	OPB_RNW			: in	std_logic;
	OPB_select		: in	std_logic;
	OPB_seqAddr		: in	std_logic
);

	attribute SIGIS						: string;
	attribute SIGIS of OPB_Clk			: signal is "Clk";
	attribute SIGIS of OPB_Rst			: signal is "Rst";

end entity opb_ftblock;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of opb_ftblock is

	component FTBlock is
	port (	clk					: in	std_logic;
				reset					: in	std_logic;
				configArrayNum		: in	integer range 0 to 2;
				configCellNum		: in	integer range 0 to 31;
				config				: in	std_logic_vector(11 downto 0);
				configWEn			: in	std_logic;
				outputSel0			: in	integer range 0 to 31;
				outputSel1			: in	integer range 0 to 31;
				outputSelWEn		: in	std_logic;
				failModeArrayNum	: in	integer range 0 to 2;
				failModeCellNum	: in	integer range 0 to 31;
				failMode				: in	std_logic_vector(1 downto 0);
				failModeWEn			: in	std_logic;
				ramDIn				: in  std_logic_vector(1 downto 0);
				ramWEn				: in  std_logic;
				ramDOut				: out std_logic_vector(1 downto 0);
				inputSel				: in	std_logic_vector(2 downto 0);
				vector				: in	std_logic_vector(3 downto 0);
				output				: out	std_logic_vector(5 downto 0);
				match					: out	std_logic_vector(5 downto 0);
				arrayError			: out	std_logic_vector(2 downto 0));
	end component;

	signal data_BUS2IP		: std_logic_vector(31 downto 0);
	signal data_IP2BUS		: std_logic_vector(31 downto 0);

	signal chipSel				: std_logic;

	signal outputSel0			: integer range 0 to 31;
	signal outputSel1			: integer range 0 to 31;
	signal outputSelWEn		: std_logic;

	signal failModeArrayNum	: integer range 0 to 2;
	signal failModeCellNum	: integer range 0 to 31;
	signal failModeWEn		: std_logic;

	signal configArrayNum	: integer range 0 to 2;
	signal configCellNum		: integer range 0 to 31;
	signal configWEn			: std_logic;

	signal ramWEn				: std_logic;
	signal ramDOut				: std_logic_vector(1 downto 0);

	signal inputSel			: std_logic_vector(2 downto 0);
	signal vector				: std_logic_vector(3 downto 0);
	signal output				: std_logic_vector(5 downto 0);
	signal match				: std_logic_vector(5 downto 0);
	signal arrayError			: std_logic_vector(2 downto 0);

	signal configCellNumReg	: unsigned(4 downto 0);
	signal vectorReg			: unsigned(3 downto 0);
	signal arrayErrorReg		: std_logic_vector(2 downto 0);

	signal subAddress			: std_logic_vector(0 to 3);

	function reverseVector (a: in std_logic_vector) return std_logic_vector is
		variable result: std_logic_vector(a'reverse_range);
	begin
		for i in a'range loop
			result((a'length-1)-i) := a(i);
 		end loop;
		return result;
	end;

begin

ledPort <= not arrayErrorReg;

chipSel		<= '1' when (OPB_select='1' and OPB_ABus(0 to 25)=C_BASEADDR(0 to 25)) else '0';

Sl_DBus		<= reverseVector(data_IP2BUS);
data_BUS2IP	<= reverseVector(OPB_DBus);

Sl_errAck	<= '0';
Sl_retry		<= '0';
Sl_toutSup	<= '0';

subAddress	<= OPB_ABus(26 to 29);

configCellNum <= to_integer(configCellNumReg);
outputSel1 <= to_integer(unsigned(data_BUS2IP(9 downto 5)));
outputSel0 <= to_integer(unsigned(data_BUS2IP(4 downto 0)));
vector <= std_logic_vector(vectorReg);

FTBlk : FTBlock
port map(clk					=> OPB_Clk,
			reset					=> OPB_Rst,
			configArrayNum		=> configArrayNum,
			configCellNum		=> configCellNum,
			config				=> data_BUS2IP(11 downto 0),
			configWEn			=> configWEn,
			outputSel0			=> outputSel0,
			outputSel1			=> outputSel1,
			outputSelWEn		=> outputSelWEn,
			failModeArrayNum	=> failModeArrayNum,
			failModeCellNum	=> failModeCellNum,
			failMode				=> data_BUS2IP(1 downto 0),
			failModeWEn			=> failModeWEn,
			ramDIn				=> data_BUS2IP(1 downto 0),
			ramWEn				=> ramWEn,
			ramDOut				=> ramDOut,
			inputSel				=> inputSel,
			vector				=> vector,
			output				=> output,
			match					=> match,
			arrayError			=> arrayError);


ReadWriteControl : process(OPB_RNW, chipSel, subAddress,
									configArrayNum, configCellNumReg, 
									outputSel1, outputSel0, failModeArrayNum, failModeCellNum,
									inputSel, vectorReg, ramDOut, match, output, arrayErrorReg)

	variable data_IP2BUS_var	: std_logic_vector(31 downto 0);
	variable Sl_xferAck_var		: std_logic;

	variable configWEn_var		: std_logic;
	variable outputSelWEn_var	: std_logic;
	variable failModeWEn_var	: std_logic;
	variable ramWEn_var			: std_logic;
begin

	data_IP2BUS_var	:= (others => '0');
	Sl_xferAck_var		:= '0';

	configWEn_var		:= '0';
	outputSelWEn_var	:= '0';
	failModeWEn_var	:= '0';
	ramWEn_var			:= '0';
	
	-- Read
	if (OPB_RNW='1' and chipSel='1') then
		case subAddress is
			when x"0"	=> data_IP2BUS_var(1 downto 0)	:= std_logic_vector(to_unsigned(configArrayNum, 2));
			when x"1"	=> data_IP2BUS_var(4 downto 0)	:= std_logic_vector(configCellNumReg);

			when x"4"	=> data_IP2BUS_var(1 downto 0)	:= std_logic_vector(to_unsigned(failModeArrayNum, 2));
			when x"5"	=> data_IP2BUS_var(4 downto 0)	:= std_logic_vector(to_unsigned(failModeCellNum, 5));

			when x"7"	=> data_IP2BUS_var(2 downto 0)	:= inputSel;
			when x"8"	=> data_IP2BUS_var(3 downto 0)	:= std_logic_vector(vectorReg);
			when x"9"	=> data_IP2BUS_var(1 downto 0)	:= ramDOut;

			when x"A"	=> data_IP2BUS_var(11 downto 0)	:= match & output;
			when x"B"	=> data_IP2BUS_var(2 downto 0)	:= arrayErrorReg;
			when others	=> null;
		end case;
		Sl_xferAck_var	:= '1';
	end if;

	-- Write
	if (OPB_RNW='0' and chipSel='1') then
		case subAddress is
			when x"2"	=> configWEn_var		:= '1';
			when x"3"	=> outputSelWEn_var	:= '1';
			when x"6"	=> failModeWEn_var	:= '1';
			when x"9"	=> ramWEn_var			:= '1';
			when others	=> null;
		end case;
		Sl_xferAck_var	:= '1';
	end if;

	data_IP2BUS		<= data_IP2BUS_var;
	Sl_xferAck		<= Sl_xferAck_var;

	configWEn		<= configWEn_var;
	outputSelWEn	<= outputSelWEn_var;
	failModeWEn		<= failModeWEn_var;
	ramWEn			<= ramWEn_var;

end process ReadWriteControl;

WriteControl : process(OPB_Clk)
begin
	if (OPB_Clk'event and OPB_Clk='1') then
		if (OPB_Rst='1') then
			configArrayNum		<= 0;
			configCellNumReg	<= (others => '0');

			failModeArrayNum	<= 0;
			failModeCellNum	<= 0;

			vectorReg			<= (others => '0');
			inputSel				<= (others => '0');

			arrayErrorReg		<= (others => '0');

		else
			arrayErrorReg <= arrayError or arrayErrorReg;

			if (OPB_RNW='0' and chipSel='1') then
				case subAddress is
					when x"0"	=> configArrayNum <= to_integer(unsigned(data_BUS2IP(1 downto 0)));
					when x"1"	=> configCellNumReg <= unsigned(data_BUS2IP(4 downto 0));
					when x"2"	=> configCellNumReg <= configCellNumReg + 1;

					when x"4"	=> failModeArrayNum <= to_integer(unsigned(data_BUS2IP(1 downto 0)));
					when x"5"	=> failModeCellNum <= to_integer(unsigned(data_BUS2IP(4 downto 0)));

					when x"7"	=> inputSel	<= data_BUS2IP(2 downto 0);
					when x"8"	=> vectorReg <= unsigned(data_BUS2IP(3 downto 0));
					when x"9"	=> vectorReg <= vectorReg + 1;

					when x"B"	=> arrayErrorReg <= (others => '0');
					when others	=> null;
				end case;
			end if;

		end if;
	end if;
end process WriteControl;

end IMP;

